
/**
 * Module dependencies.
 */

var express = require('express')
  , http = require('http')
  , path = require('path')
  , fs = require("fs")
  , _ = require('underscore');
var upload = require('jquery-file-upload-middleware');
var app = module.exports = express();

// commandline option
// multiuser mode
var isMultiuser = (_.indexOf(process.argv, "multiuser") !== -1)? true: false;
var isAllowRemote = (_.indexOf(process.argv, "allowremote") !== -1)? true: false;
app.set('isMultiuser', isMultiuser);
app.set('isAllowRemote', isAllowRemote);
if(!isMultiuser && !fs.existsSync("userdata/networks.csv")){
    fs.writeFileSync("userdata/networks.csv", "");
}
var routes = require('./routes');
routes.file = require('./routes/file.js');


if(isMultiuser){
    MySQLSessionStore = require('connect-mysql-session')(express);
    // db setup
    var dbsetup = {
        host        : 'localhost',    // hostname of MySQL server
        dbname      : "harkdesigner", // name of the database that hark-designer uses.
        dbuser      : "harkdesigner", // username of harkdesigner.
        dbpassword  : null            // password of harkdesigner.
    };
    app.set('dbsetup', dbsetup);
    routes.multiuser = require('./routes/multiuser.js');
}

// all environments
app.set('port', process.env.PORT || 3000);
app.set('views', __dirname + '/views');
app.set('view engine', 'jade');

// upload.configure();

// for multiuser
if(isMultiuser){
  app.use(express.cookieParser());
  app.use(express.cookieSession({
      secret: 'secret',
      store: new MySQLSessionStore (
          dbsetup.dbname, dbsetup.dbuser, dbsetup.dbpassword, {logging: false}),
      cookie: {
          httpOnly: false,
          maxAge: new Date(Date.now() + 60 * 60 * 1000)
      },
      // forceSync: true
  }));
}

var logStream;
if(isMultiuser){
    logStream = fs.createWriteStream('userdata/access_log', {flags: 'a'});
} else{
    logStream = fs.createWriteStream('userdata/access_log', {flags: 'w'});
}
app.use(express.logger({stream: logStream}));
// app.use('/file/:type', upload.fileHandler());

app.use(express.json());
app.use(express.urlencoded());
app.use(express.methodOverride());

// access restriction
app.use(function(req, res, next){
    var hostname = req.headers.host;
    hostname = hostname.split(":")[0];
    if(hostname  === null || hostname === undefined){
        res.send(400);
        return;
    }
    // if the request is from localhost, accept.
    if(hostname === "localhost" || hostname === "127.0.0.1"){
        next();
        return;
    }
    // if the "isAllowRemote" is true or multiuser mode, accept remote access.
    else if(isAllowRemote || isMultiuser){
        next();
        return;
    }
    // if the remote access is not allowed, respond 400.
    else{
        res.send(400);
    }
});

app.put('/file/object/:name', routes.file.saveNetwork);
app.post('/file/:type/:name',   routes.file.addFile);
app.post('/file/:type/:name/:size',   routes.file.addFile);
app.post('/package/:name', routes.file.addPackage);
app.del('/package/:pkgname', routes.file.delPackage);
app.get('/package', routes.file.existingPackage);

app.use(app.router);
app.use(express.static(path.join(__dirname, 'public')));
app.use(express.favicon(path.join(__dirname, 'public/favicon.ico')));
app.set("dirname", __dirname);
console.log(__dirname);

// development only
if ('development' == app.get('env')) {
  app.use(express.errorHandler());
}

app.get('/', routes.index);
app.get('/keepalive', routes.keepalive);
app.get('/def/:name', routes.nodedefs);
app.get('/preference/:file', routes.preference)
app.put('/preference/:file', routes.preferenceSet)

app.get('/file/:name', routes.file.getFile);
app.del('/file/:name', routes.file.deleteFile);
app.get("/filelist", routes.file.getAllFiles);
app.get("/dl/:type/:name", routes.file.dlFile);
app.get("/datapath", routes.file.datapath);

app.get("/execution/:file", routes.execution);
app.get("/abort", routes.abort);
app.get("/execstatus", routes.execstatus);

app.get("/packagelist", routes.packagelist);

if(isMultiuser){
  app.post("/login/:type", routes.multiuser.checklogin);
  app.get("/login", routes.multiuser.login);
  app.get("/logout", routes.multiuser.logout);
  app.get("/admin", routes.multiuser.admin);
  app.get("/admin/userlist", routes.multiuser.allusers);  
  app.del("/admin/user/:name", routes.multiuser.deluser);
  app.post("/admin/user/:name", routes.multiuser.updateuser);
}

app.get("/os", routes.os);
app.put('/reshapedNodes', routes.reshape)

http.createServer(app).listen(app.get('port'), function(){
  console.log('Express server listening on port ' + app.get('port'));
});
